import sys
from collections import defaultdict

import networkx as nx


def in_correct_order(update: list[int], requirements: dict[int, set[int]]) -> bool:
    for i_value, value in enumerate(update):
        if any(value_2 in requirements[value] for value_2 in update[i_value:]):
            return False
    return True


def fix_order(update: list[int], requirements: dict[int, set[int]]) -> list[int]:
    graph = nx.DiGraph()
    graph.add_nodes_from(update)
    graph.add_edges_from(
        (v, k)
        for k, vs in requirements.items()
        for v in vs
        if k in update and v in update
    )

    return list(nx.topological_sort(graph))


part1, part2 = sys.stdin.read().split("\n\n")

requirements: dict[int, set[int]] = defaultdict(set)
for line in part1.splitlines():
    v1, v2 = line.split("|")
    requirements[int(v2)].add(int(v1))

updates = [list(map(int, line.split(","))) for line in part2.splitlines()]

answer_1 = sum(
    update[len(update) // 2]
    for update in updates
    if in_correct_order(update, requirements)
)

answer_2 = sum(
    fix_order(update, requirements)[len(update) // 2]
    for update in updates
    if not in_correct_order(update, requirements)
)

print(f"answer 1 is {answer_1}")
print(f"answer 2 is {answer_2}")
